@@ -42,6 +42,12 @@ static GSList *queued_position_saves = NULL;
 static guint   queued_position_source = 0;
 
 
+static inline PanelWidget *
+panel_applet_get_panel_widget (AppletInfo *info)
+{
+	return PANEL_WIDGET (gtk_widget_get_parent (info->widget));
+}
+
 static void
 panel_applet_set_dnd_enabled (AppletInfo *info,
 			      gboolean    dnd_enabled)
@@ -57,7 +63,7 @@ panel_applet_set_dnd_enabled (AppletInfo *info,
 	case PANEL_OBJECT_LAUNCHER:
 		panel_launcher_set_dnd_enabled (info->data, dnd_enabled);
 		break;
-	case PANEL_OBJECT_BONOBO:
+	case PANEL_OBJECT_APPLET:
 		break;
 	case PANEL_OBJECT_LOGOUT:
 	case PANEL_OBJECT_LOCK:
@@ -81,7 +87,7 @@ panel_applet_toggle_locked (AppletInfo *info)
 	PanelWidget *panel_widget;
 	gboolean     locked;
 
-	panel_widget = PANEL_WIDGET (info->widget->parent);
+	panel_widget = panel_applet_get_panel_widget (info);
 	
 	locked = panel_widget_toggle_applet_locked (panel_widget, info->widget);
 
@@ -108,14 +114,18 @@ panel_applet_lock (GtkCheckMenuItem *menuitem,
 static void
 move_applet_callback (GtkWidget *widget, AppletInfo *info)
 {
+	GtkWidget   *parent;
 	PanelWidget *panel;
 
 	g_return_if_fail (info != NULL);
 	g_return_if_fail (info->widget != NULL);
-	g_return_if_fail (info->widget->parent != NULL);
-	g_return_if_fail (PANEL_IS_WIDGET (info->widget->parent));
 
-	panel = PANEL_WIDGET (info->widget->parent);
+	parent = gtk_widget_get_parent (info->widget);
+
+	g_return_if_fail (parent != NULL);
+	g_return_if_fail (PANEL_IS_WIDGET (parent));
+
+	panel = PANEL_WIDGET (parent);
 
 	panel_widget_applet_drag_start (panel, info->widget,
 					PW_DRAG_OFF_CENTER,
@@ -185,7 +195,7 @@ panel_applet_locked_change_notify (GConfClient *client,
 
 	locked = gconf_value_get_bool (value);
 
-	panel_widget = PANEL_WIDGET (info->widget->parent);
+	panel_widget = panel_applet_get_panel_widget (info);
 	applet_locked = panel_widget_get_applet_locked (panel_widget,
 							info->widget);
 
@@ -194,7 +204,7 @@ panel_applet_locked_change_notify (GConfClient *client,
 
 	panel_applet_toggle_locked (info);
 
-	if (info->type == PANEL_OBJECT_BONOBO)
+	if (info->type == PANEL_OBJECT_APPLET)
 		panel_applet_frame_sync_menu_state (PANEL_APPLET_FRAME (info->widget));
 	else
 		panel_applet_recreate_menu (info);
@@ -216,7 +226,7 @@ applet_user_menu_get_screen (AppletUserMenu *menu)
 {
 	PanelWidget *panel_widget;
 
-	panel_widget = PANEL_WIDGET (menu->info->widget->parent);
+	panel_widget = panel_applet_get_panel_widget (menu->info);
 
 	return gtk_window_get_screen (GTK_WINDOW (panel_widget->toplevel));
 }
@@ -268,7 +278,7 @@ applet_callback_callback (GtkWidget      *widget,
 			PANEL_MENU_BAR (menu->info->widget), menu->name);
 		break;
 
-	case PANEL_OBJECT_BONOBO:
+	case PANEL_OBJECT_APPLET:
 		/*
 		 * Applet's menu's are handled differently
 		 */
@@ -287,7 +297,7 @@ applet_menu_show (GtkWidget *w,
 {
 	PanelWidget *panel_widget;
 
-	panel_widget = PANEL_WIDGET (info->widget->parent);
+	panel_widget = panel_applet_get_panel_widget (info);
 
 	panel_toplevel_push_autohide_disabler (panel_widget->toplevel);
 }
@@ -299,7 +309,7 @@ applet_menu_deactivate (GtkWidget *w,
 {
 	PanelWidget *panel_widget;
 
-	panel_widget = PANEL_WIDGET (info->widget->parent);
+	panel_widget = panel_applet_get_panel_widget (info);
 
 	panel_toplevel_pop_autohide_disabler (panel_widget->toplevel);
 }
@@ -470,7 +480,7 @@ panel_applet_create_menu (AppletInfo *info)
 	PanelWidget *panel_widget;
 	gboolean     added_anything = FALSE;
 
-	panel_widget = PANEL_WIDGET (info->widget->parent);
+	panel_widget = panel_applet_get_panel_widget (info);
 
 	menu = g_object_ref_sink (gtk_menu_new ());
 
@@ -574,7 +584,7 @@ panel_applet_menu_set_recurse (GtkMenu     *menu,
 	children = gtk_container_get_children (GTK_CONTAINER (menu));
 
 	for (l = children; l; l = l->next) {
-		GtkWidget *submenu = GTK_MENU_ITEM (l->data)->submenu;
+		GtkWidget *submenu = gtk_menu_item_get_submenu (GTK_MENU_ITEM (l->data));
 
 		if (submenu)
 			panel_applet_menu_set_recurse (
@@ -591,39 +601,45 @@ panel_applet_position_menu (GtkMenu   *menu,
 			    gboolean  *push_in,
 			    GtkWidget *applet)
 {
+	GtkAllocation   allocation;
 	GtkRequisition  requisition;
 	GdkScreen      *screen;
+	GtkWidget      *parent;
 	int             menu_x = 0;
 	int             menu_y = 0;
 	int             pointer_x;
 	int             pointer_y;
 
-	g_return_if_fail (PANEL_IS_WIDGET (applet->parent));
+	parent = gtk_widget_get_parent (applet);
+
+	g_return_if_fail (PANEL_IS_WIDGET (parent));
 
 	screen = gtk_widget_get_screen (applet);
 
 	gtk_widget_size_request (GTK_WIDGET (menu), &requisition);
 
-	gdk_window_get_origin (applet->window, &menu_x, &menu_y);
+	gdk_window_get_origin (gtk_widget_get_window (applet), &menu_x, &menu_y);
 	gtk_widget_get_pointer (applet, &pointer_x, &pointer_y);
 
+	gtk_widget_get_allocation (applet, &allocation);
+
 	if (!gtk_widget_get_has_window (applet)) {
-		menu_x += applet->allocation.x;
-		menu_y += applet->allocation.y;
+		menu_x += allocation.x;
+		menu_y += allocation.y;
 	}
 
-	if (PANEL_WIDGET (applet->parent)->orient == GTK_ORIENTATION_HORIZONTAL) {
+	if (PANEL_WIDGET (parent)->orient == GTK_ORIENTATION_HORIZONTAL) {
 		if (gtk_widget_get_direction (GTK_WIDGET (menu)) != GTK_TEXT_DIR_RTL) {
-			if (pointer_x < applet->allocation.width &&
+			if (pointer_x < allocation.width &&
 			    requisition.width < pointer_x)
 				menu_x += MIN (pointer_x,
-					       applet->allocation.width - requisition.width);
+					       allocation.width - requisition.width);
 		} else {
-			menu_x += applet->allocation.width - requisition.width;
-			if (pointer_x > 0 && pointer_x < applet->allocation.width &&
-			    pointer_x < applet->allocation.width - requisition.width) {
-				menu_x -= MIN (applet->allocation.width - pointer_x,
-					       applet->allocation.width - requisition.width);
+			menu_x += allocation.width - requisition.width;
+			if (pointer_x > 0 && pointer_x < allocation.width &&
+			    pointer_x < allocation.width - requisition.width) {
+				menu_x -= MIN (allocation.width - pointer_x,
+					       allocation.width - requisition.width);
 			}
 		}
 		menu_x = MIN (menu_x, gdk_screen_get_width (screen) - requisition.width);
@@ -631,17 +647,17 @@ panel_applet_position_menu (GtkMenu   *menu,
 		if (menu_y > gdk_screen_get_height (screen) / 2)
 			menu_y -= requisition.height;
 		else
-			menu_y += applet->allocation.height;
+			menu_y += allocation.height;
 	} else {
-		if (pointer_y < applet->allocation.height &&
+		if (pointer_y < allocation.height &&
 		    requisition.height < pointer_y)
-			menu_y += MIN (pointer_y, applet->allocation.height - requisition.height);
+			menu_y += MIN (pointer_y, allocation.height - requisition.height);
 		menu_y = MIN (menu_y, gdk_screen_get_height (screen) - requisition.height);
 
 		if (menu_x > gdk_screen_get_width (screen) / 2)
 			menu_x -= requisition.width;
 		else
-			menu_x += applet->allocation.width;
+			menu_x += allocation.width;
 	}
 
 	*x = menu_x;
@@ -657,7 +673,7 @@ applet_show_menu (AppletInfo     *info,
 
 	g_return_if_fail (info != NULL);
 
-	panel_widget = PANEL_WIDGET (info->widget->parent);
+	panel_widget = panel_applet_get_panel_widget (info);
 
 	if (info->menu == NULL)
 		info->menu = panel_applet_create_menu (info);
@@ -667,7 +683,7 @@ applet_show_menu (AppletInfo     *info,
 
 	panel_applet_menu_set_recurse (GTK_MENU (info->menu),
 				       "menu_panel",
-				       info->widget->parent);
+				       panel_widget);
 
 	gtk_menu_set_screen (GTK_MENU (info->menu),
 			     gtk_window_get_screen (GTK_WINDOW (panel_widget->toplevel)));
@@ -692,7 +708,7 @@ applet_do_popup_menu (GtkWidget      *widget,
 	if (panel_applet_is_in_drag ())
 		return FALSE;
 
-	if (info->type == PANEL_OBJECT_BONOBO)
+	if (info->type == PANEL_OBJECT_APPLET)
 		return FALSE;
 
 	applet_show_menu (info, event);
@@ -753,7 +769,7 @@ panel_applet_destroy (GtkWidget  *widget,
 		}
 	}
 
-	if (info->type != PANEL_OBJECT_BONOBO)
+	if (info->type != PANEL_OBJECT_APPLET)
 		panel_lockdown_notify_remove (G_CALLBACK (panel_applet_recreate_menu),
 					      info);
 
@@ -825,6 +841,11 @@ panel_applet_on_load_queue (const char *id)
 		if (strcmp (applet->id, id) == 0)
 			return TRUE;
 	}
+	for (li = panel_applets_loading; li != NULL; li = li->next) {
+		PanelAppletToLoad *applet = li->data;
+		if (strcmp (applet->id, id) == 0)
+			return TRUE;
+	}
 	return FALSE;
 }
 
@@ -872,6 +893,7 @@ panel_applet_stop_loading (const char *id)
 static gboolean
 panel_applet_load_idle_handler (gpointer dummy)
 {
+	PanelObjectType    applet_type;
 	PanelAppletToLoad *applet = NULL;
 	PanelToplevel     *toplevel = NULL;
 	PanelWidget       *panel_widget;
@@ -891,7 +913,7 @@ panel_applet_load_idle_handler (gpointer dummy)
 	}
 
 	if (!l) {
-		/* All the rest of the applets don't have a panel */
+		/* All the remaining applets don't have a panel */
 		for (l = panel_applets_to_load; l; l = l->next)
 			free_applet_to_load (l->data);
 		g_slist_free (panel_applets_to_load);
@@ -918,8 +940,16 @@ panel_applet_load_idle_handler (gpointer dummy)
 			applet->position = -1;
 	}
 
-	switch (applet->type) {
-	case PANEL_OBJECT_BONOBO:
+	/* We load applets asynchronously, so we specifically don't call
+	 * panel_applet_stop_loading() for this type. However, in case of
+	 * failure during the load, we might call panel_applet_stop_loading()
+	 * synchronously, which will make us lose the content of the applet
+	 * variable. So we save the type to be sure we always ignore the
+	 * applets. */
+	applet_type = applet->type;
+
+	switch (applet_type) {
+	case PANEL_OBJECT_APPLET:
 		panel_applet_frame_load_from_gconf (
 					panel_widget,
 					applet->locked,
@@ -982,8 +1012,8 @@ panel_applet_load_idle_handler (gpointer dummy)
 		break;
 	}
 
-	/* Only the bonobo applets will do a late stop_loading */
-	if (applet->type != PANEL_OBJECT_BONOBO)
+	/* Only the real applets will do a late stop_loading */
+	if (applet_type != PANEL_OBJECT_APPLET)
 		panel_applet_stop_loading (applet->id);
 
 	return TRUE;
@@ -1038,7 +1068,7 @@ panel_applet_load_queued_applets (gboolean initial_load)
 		return;
 	}
 
-	if (initial_load && panel_applets_to_load) {
+	if (panel_applets_to_load && panel_applet_unhide_toplevels_timeout == 0) {
 		/* Install a timeout to make sure we don't block the
 		 * unhiding because of an applet that doesn't load */
 		panel_applet_unhide_toplevels_timeout =
@@ -1073,7 +1103,7 @@ panel_applet_get_toplevel_id (AppletInfo *applet)
 	g_return_val_if_fail (applet != NULL, NULL);
 	g_return_val_if_fail (GTK_IS_WIDGET (applet->widget), NULL);
 
-	panel_widget = PANEL_WIDGET (applet->widget->parent);
+	panel_widget = panel_applet_get_panel_widget (applet);
 	if (!panel_widget)
 		return NULL;
 
@@ -1135,9 +1165,9 @@ panel_applet_save_position (AppletInfo *applet_info,
 
 	client  = panel_gconf_get_client ();
 
-	key_type = applet_info->type == PANEL_OBJECT_BONOBO ? PANEL_GCONF_APPLETS : PANEL_GCONF_OBJECTS;
+	key_type = applet_info->type == PANEL_OBJECT_APPLET ? PANEL_GCONF_APPLETS : PANEL_GCONF_OBJECTS;
 	
-	panel_widget = PANEL_WIDGET (applet_info->widget->parent);
+	panel_widget = panel_applet_get_panel_widget (applet_info);
 
 	/* FIXME: Instead of getting keys, comparing and setting, there
 	   should be a dirty flag */
@@ -1277,11 +1307,11 @@ panel_applet_register (GtkWidget       *applet,
 
 	g_object_set_data (G_OBJECT (applet), "applet_info", info);
 
-	if (type != PANEL_OBJECT_BONOBO)
+	if (type != PANEL_OBJECT_APPLET)
 		panel_lockdown_notify_add (G_CALLBACK (panel_applet_recreate_menu),
 					   info);
 
-	key = panel_gconf_full_key ((type == PANEL_OBJECT_BONOBO) ?
+	key = panel_gconf_full_key ((type == PANEL_OBJECT_APPLET) ?
 				     PANEL_GCONF_APPLETS : PANEL_GCONF_OBJECTS,
 				    id, "locked");
 	panel_gconf_notify_add_while_alive (key,
@@ -1347,7 +1377,7 @@ panel_applet_register (GtkWidget       *applet,
 	size_change (info, panel);
 	back_change (info, panel);
 
-	if (type != PANEL_OBJECT_BONOBO)
+	if (type != PANEL_OBJECT_APPLET)
 		gtk_widget_grab_focus (applet);
 	else
 		gtk_widget_child_focus (applet, GTK_DIR_TAB_FORWARD);
@@ -1380,7 +1410,7 @@ panel_applet_can_freely_move (AppletInfo *applet)
 
 	client  = panel_gconf_get_client ();
 	
-	key_type = (applet->type == PANEL_OBJECT_BONOBO) ? PANEL_GCONF_APPLETS : PANEL_GCONF_OBJECTS;
+	key_type = (applet->type == PANEL_OBJECT_APPLET) ? PANEL_GCONF_APPLETS : PANEL_GCONF_OBJECTS;
        
 	key = panel_gconf_full_key (key_type, applet->id, "position");
 	if (!gconf_client_key_is_writable (client, key, NULL))
@@ -1409,7 +1439,7 @@ panel_applet_lockable (AppletInfo *applet)
 	
 	client  = panel_gconf_get_client ();
 	
-	key_type = (applet->type == PANEL_OBJECT_BONOBO) ? PANEL_GCONF_APPLETS : PANEL_GCONF_OBJECTS;
+	key_type = (applet->type == PANEL_OBJECT_APPLET) ? PANEL_GCONF_APPLETS : PANEL_GCONF_OBJECTS;
 
 	key = panel_gconf_full_key (key_type, applet->id, "locked");
 
