@@ -32,6 +32,7 @@
 #include <gconf/gconf-client.h>
 
 #include <libpanel-util/panel-keyfile.h>
+#include <libpanel-util/panel-xdg.h>
 
 #include "launcher.h"
 #include "panel-util.h"
@@ -127,7 +128,7 @@ menu_get_panel (GtkWidget *menu)
 	g_return_val_if_fail (menu != NULL, NULL);
 
 	if (GTK_IS_MENU_ITEM (menu))
-		menu = menu->parent;
+		menu = gtk_widget_get_parent (menu);
 
 	g_return_val_if_fail (GTK_IS_MENU (menu), NULL);
 
@@ -136,7 +137,7 @@ menu_get_panel (GtkWidget *menu)
 		if (retval)
 			break;
 
-		menu = gtk_menu_get_attach_widget (GTK_MENU (menu))->parent;
+		menu = gtk_widget_get_parent (gtk_menu_get_attach_widget (GTK_MENU (menu)));
 		if (!GTK_IS_MENU (menu))
 			break;
 	}
@@ -190,7 +191,7 @@ reload_image_menu_items (void)
 		GtkWidget *image = l->data;
 		gboolean   is_mapped;
       
-		is_mapped = GTK_WIDGET_MAPPED (image);
+		is_mapped = gtk_widget_get_mapped (image);
 
 		if (is_mapped)
 			gtk_widget_unmap (image);
@@ -444,7 +445,7 @@ menu_item_style_set (GtkImage *image,
 
 	widget = GTK_WIDGET (image);
 
-	is_mapped = GTK_WIDGET_MAPPED (widget);
+	is_mapped = gtk_widget_get_mapped (widget);
 	if (is_mapped)
 		gtk_widget_unmap (widget);
 
@@ -514,7 +515,7 @@ load_icons_handler_again:
 	icons_to_load = g_list_delete_link (icons_to_load, icons_to_load);
 
 	/* if not visible anymore, just ignore */
-	if ( ! GTK_WIDGET_VISIBLE (icon->pixmap)) {
+	if ( ! gtk_widget_get_visible (icon->pixmap)) {
 		icon_to_load_free (icon);
 		/* we didn't do anything long/hard, so just do this again,
 		 * this is fun, don't go back to main loop */
@@ -813,7 +814,7 @@ static void
 restore_grabs(GtkWidget *w, gpointer data)
 {
 	GtkWidget *menu_item = data;
-	GtkMenu *menu = GTK_MENU(menu_item->parent); 
+	GtkMenu *menu = GTK_MENU (gtk_widget_get_parent (menu_item));
 	GtkWidget *xgrab_shell;
 	GtkWidget *parent;
 
@@ -826,11 +827,11 @@ restore_grabs(GtkWidget *w, gpointer data)
 		GtkWidget *tmp = parent;
 
 		while (tmp) {
-			if (!GTK_WIDGET_MAPPED (tmp)) {
+			if (!gtk_widget_get_mapped (tmp)) {
 				viewable = FALSE;
 				break;
 			}
-			tmp = tmp->parent;
+			tmp = gtk_widget_get_parent (tmp);
 		}
 
 		if (viewable)
@@ -842,14 +843,16 @@ restore_grabs(GtkWidget *w, gpointer data)
 	/*only grab if this HAD a grab before*/
 	if (xgrab_shell && (GTK_MENU_SHELL (xgrab_shell)->have_xgrab))
           {
-	    if (gdk_pointer_grab (xgrab_shell->window, TRUE,
+	    GdkWindow *window = gtk_widget_get_window (xgrab_shell);
+
+	    if (gdk_pointer_grab (window, TRUE,
 				  GDK_BUTTON_PRESS_MASK |
 				  GDK_BUTTON_RELEASE_MASK |
 				  GDK_ENTER_NOTIFY_MASK |
 				  GDK_LEAVE_NOTIFY_MASK,
 				  NULL, NULL, 0) == 0)
               {
-		if (gdk_keyboard_grab (xgrab_shell->window, TRUE,
+		if (gdk_keyboard_grab (window, TRUE,
 				       GDK_CURRENT_TIME) == 0)
 		  GTK_MENU_SHELL (xgrab_shell)->have_xgrab = TRUE;
 		else
@@ -1023,7 +1026,7 @@ drag_end_menu_cb (GtkWidget *widget, GdkDragContext     *context)
 
   /* Find the last viewable ancestor, and make an X grab on it
    */
-  parent = widget->parent;
+  parent = gtk_widget_get_parent (widget);
   xgrab_shell = NULL;
 
   /* FIXME: workaround for a possible gtk+ bug
@@ -1038,12 +1041,12 @@ drag_end_menu_cb (GtkWidget *widget, GdkDragContext     *context)
       
       while (tmp)
 	{
-	  if (!GTK_WIDGET_MAPPED (tmp))
+	  if (!gtk_widget_get_mapped (tmp))
 	    {
 	      viewable = FALSE;
 	      break;
 	    }
-	  tmp = tmp->parent;
+	  tmp = gtk_widget_get_parent (tmp);
 	}
       
       if (viewable)
@@ -1052,17 +1055,18 @@ drag_end_menu_cb (GtkWidget *widget, GdkDragContext     *context)
       parent = GTK_MENU_SHELL (parent)->parent_menu_shell;
     }
   
-  if (xgrab_shell && !GTK_MENU(xgrab_shell)->torn_off)
+  if (xgrab_shell && !gtk_menu_get_tearoff_state (GTK_MENU(xgrab_shell)))
     {
+      GdkWindow *window = gtk_widget_get_window (xgrab_shell);
       GdkCursor *cursor = gdk_cursor_new (GDK_ARROW);
 
-      if ((gdk_pointer_grab (xgrab_shell->window, TRUE,
+      if ((gdk_pointer_grab (window, TRUE,
 			     GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK |
 			     GDK_ENTER_NOTIFY_MASK | GDK_LEAVE_NOTIFY_MASK |
 			     GDK_POINTER_MOTION_MASK,
 			     NULL, cursor, GDK_CURRENT_TIME) == 0))
 	{
-	  if (gdk_keyboard_grab (xgrab_shell->window, TRUE,
+	  if (gdk_keyboard_grab (window, TRUE,
 				 GDK_CURRENT_TIME) == 0)
 	    GTK_MENU_SHELL (xgrab_shell)->have_xgrab = TRUE;
 	  else
@@ -1093,7 +1097,7 @@ drag_data_get_menu_cb (GtkWidget        *widget,
 	g_free (uri);
 
 	gtk_selection_data_set (selection_data,
-				selection_data->target, 8, (guchar *)uri_list,
+				gtk_selection_data_get_target (selection_data), 8, (guchar *)uri_list,
 				strlen (uri_list));
 	g_free (uri_list);
 }
@@ -1117,8 +1121,8 @@ image_menuitem_size_request (GtkWidget      *menuitem,
 	 * gtk_menu_item_size_request()
 	 */
 	req_height = icon_height;
-	req_height += (GTK_CONTAINER (menuitem)->border_width +
-		       menuitem->style->ythickness) * 2;
+	req_height += (gtk_container_get_border_width (GTK_CONTAINER (menuitem)) +
+		       (gtk_widget_get_style (menuitem))->ythickness) * 2;
 	requisition->height = MAX (requisition->height, req_height);
 }
 
@@ -1205,7 +1209,7 @@ drag_data_get_string_cb (GtkWidget *widget, GdkDragContext     *context,
 			 guint time, const char *string)
 {
 	gtk_selection_data_set (selection_data,
-				selection_data->target, 8, (guchar *)string,
+				gtk_selection_data_get_target (selection_data), 8, (guchar *)string,
 				strlen(string));
 }
 
@@ -1388,8 +1392,16 @@ create_submenu_entry (GtkWidget          *menu,
 		      GMenuTreeDirectory *directory)
 {
 	GtkWidget *menuitem;
+	gboolean   force_categories_icon;
+
+	force_categories_icon = g_object_get_data (G_OBJECT (menu),
+						   "panel-menu-force-icon-for-categories") != NULL;
+
+	if (force_categories_icon)
+		menuitem = panel_image_menu_item_new ();
+	else
+		menuitem = gtk_image_menu_item_new ();
 
-	menuitem = gtk_image_menu_item_new ();
 	panel_load_menu_image_deferred (menuitem,
 					panel_menu_icon_get_size (),
 					NULL, NULL,
@@ -1415,6 +1427,7 @@ create_submenu (GtkWidget          *menu,
 {
 	GtkWidget *menuitem;
 	GtkWidget *submenu;
+	gboolean   force_categories_icon;
 
 	if (alias_directory)
 		menuitem = create_submenu_entry (menu, alias_directory);
@@ -1424,6 +1437,13 @@ create_submenu (GtkWidget          *menu,
 	submenu = create_fake_menu (directory);
 
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem), submenu);
+
+	/* Keep the infor that we force (or not) the icons to be visible */
+	force_categories_icon = g_object_get_data (G_OBJECT (menu),
+						   "panel-menu-force-icon-for-categories") != NULL;
+	g_object_set_data (G_OBJECT (submenu),
+			   "panel-menu-force-icon-for-categories",
+			   GINT_TO_POINTER (force_categories_icon));
 }
 
 static void 
@@ -1479,16 +1499,20 @@ create_menuitem (GtkWidget          *menu,
 			panel_menu_icon_get_size (),
 			NULL,
 			alias_directory ? gmenu_tree_directory_get_name (alias_directory) :
-					  gmenu_tree_entry_get_name (entry));
+					  gmenu_tree_entry_get_display_name (entry));
 
-	if ((alias_directory &&
-	     gmenu_tree_directory_get_comment (alias_directory)) ||
-	    (!alias_directory &&
-	     gmenu_tree_entry_get_comment (entry)))
+	if (alias_directory &&
+	    gmenu_tree_directory_get_comment (alias_directory))
 		panel_util_set_tooltip_text (menuitem,
-					     alias_directory ?
-						gmenu_tree_directory_get_comment (alias_directory) :
-						gmenu_tree_entry_get_comment (entry));
+					     gmenu_tree_directory_get_comment (alias_directory));
+	else if	(!alias_directory &&
+		 gmenu_tree_entry_get_comment (entry))
+		panel_util_set_tooltip_text (menuitem,
+					     gmenu_tree_entry_get_comment (entry));
+	else if	(!alias_directory &&
+		 gmenu_tree_entry_get_generic_name (entry))
+		panel_util_set_tooltip_text (menuitem,
+					     gmenu_tree_entry_get_generic_name (entry));
 
 	g_signal_connect_after (menuitem, "button_press_event",
 				G_CALLBACK (menuitem_button_press_event), NULL);
@@ -1509,7 +1533,7 @@ create_menuitem (GtkWidget          *menu,
 
 			icon = gmenu_tree_entry_get_icon (entry);
 			if (!g_path_is_absolute (icon)) {
-				icon_no_ext = panel_util_icon_remove_extension (icon);
+				icon_no_ext = panel_xdg_icon_remove_extension (icon);
 				gtk_drag_source_set_icon_name (menuitem,
 							       icon_no_ext);
 				g_free (icon_no_ext);
@@ -1598,7 +1622,8 @@ remove_gmenu_tree_monitor (GtkWidget *menu,
 
 GtkWidget *
 create_applications_menu (const char *menu_file,
-			  const char *menu_path)
+			  const char *menu_path,
+			  gboolean    always_show_image)
 {
 	GMenuTree *tree;
 	GtkWidget *menu;
@@ -1606,7 +1631,13 @@ create_applications_menu (const char *menu_file,
 
 	menu = create_empty_menu ();
 
+	if (always_show_image)
+		g_object_set_data (G_OBJECT (menu),
+				   "panel-menu-force-icon-for-categories",
+				   GINT_TO_POINTER (TRUE));
+
 	tree = gmenu_tree_lookup (menu_file, GMENU_TREE_FLAGS_NONE);
+	gmenu_tree_set_sort_key (tree, GMENU_TREE_SORT_DISPLAY_NAME);
 
 	g_object_set_data_full (G_OBJECT (menu),
 				"panel-menu-tree",
@@ -1652,11 +1683,14 @@ static GtkWidget *
 populate_menu_from_directory (GtkWidget          *menu,
 			      GMenuTreeDirectory *directory)
 {	
+	GList    *children;
 	GSList   *l;
 	GSList   *items;
 	gboolean  add_separator;
 
-	add_separator = (GTK_MENU_SHELL (menu)->children != NULL);
+	children = gtk_container_get_children (GTK_CONTAINER (menu));
+	add_separator = (children != NULL);
+	g_list_free (children);
 
 	items = gmenu_tree_directory_get_contents (directory);
 
@@ -1759,7 +1793,8 @@ create_main_menu (PanelWidget *panel)
 {
 	GtkWidget *main_menu;
 
-	main_menu = create_applications_menu ("applications.menu", NULL);
+	main_menu = create_applications_menu ("applications.menu", NULL, TRUE);
+
 	g_object_set_data (G_OBJECT (main_menu), "menu_panel", panel);
 	/* FIXME need to update the panel on parent_set */
 
@@ -1829,8 +1864,7 @@ panel_load_menu_image_deferred (GtkWidget   *image_menu_item,
 	gtk_icon_size_lookup (icon_size, NULL, &icon_height);
 
 	image = gtk_image_new ();
-	image->requisition.width  = icon_height;
-	image->requisition.height = icon_height;
+	gtk_widget_set_size_request (image, icon_height, icon_height);
 
 	/* this takes over the floating ref */
 	icon->pixmap = g_object_ref_sink (G_OBJECT (image));
