@@ -25,15 +25,17 @@
 #include <config.h>
 #include <string.h>
 
-#include <libbonobo.h>
+#include <glib/gi18n.h>
 
 #include <gmenu-tree.h>
 
 #include <libpanel-util/panel-glib.h>
+#include <libpanel-util/panel-show.h>
 
 #include "launcher.h"
 #include "panel.h"
 #include "drawer.h"
+#include "panel-applets-manager.h"
 #include "panel-applet-frame.h"
 #include "panel-action-button.h"
 #include "panel-menu-bar.h"
@@ -180,16 +182,6 @@ static PanelAddtoItemInfo internal_addto_items [] = {
 	  TRUE }
 };
 
-static const char applet_requirements [] =
-	"has_all (repo_ids, ['IDL:Bonobo/Control:1.0',"
-	"		     'IDL:GNOME/Vertigo/PanelAppletShell:1.0']) && "
-	"defined (panel:icon)";
-
-static char *applet_sort_criteria [] = {
-	"name",
-	NULL
-	};
-
 enum {
 	COLUMN_ICON,
 	COLUMN_TEXT,
@@ -295,8 +287,8 @@ panel_addto_drag_data_get_cb (GtkWidget        *widget,
 			      const char       *string)
 {
 	gtk_selection_data_set (selection_data,
-				selection_data->target, 8, (guchar *) string,
-				strlen (string));
+				gtk_selection_data_get_target (selection_data), 8,
+				(guchar *) string, strlen (string));
 }
 
 static void
@@ -392,56 +384,23 @@ panel_addto_setup_internal_applet_drag (GtkTreeView *tree_view,
 static GSList *
 panel_addto_query_applets (GSList *list)
 {
-	Bonobo_ServerInfoList *applet_list;
-	CORBA_Environment   env;
-	const char * const *langs;
-	GSList             *langs_gslist;
-	int                 i;
+	GList *applet_list, *l;
 
-	CORBA_exception_init (&env);
+	applet_list = panel_applets_manager_get_applets ();
 
-	applet_list = bonobo_activation_query (applet_requirements,
-					       applet_sort_criteria,
-					       &env);
-	if (BONOBO_EX (&env)) {
-		g_warning (_("query returned exception %s\n"),
-			   BONOBO_EX_REPOID (&env));
-
-		CORBA_exception_free (&env);
-		CORBA_free (applet_list);
-
-		return NULL;
-	}
-
-	CORBA_exception_free (&env);
-
-	langs = g_get_language_names ();
-
-	langs_gslist = NULL;
-	for (i = 0; langs[i]; i++)
-		langs_gslist = g_slist_prepend (langs_gslist, (char *) langs[i]);
-
-	langs_gslist = g_slist_reverse (langs_gslist);
-
-	for (i = 0; i < applet_list->_length; i++) {
-		Bonobo_ServerInfo *info;
-		const char *name, *description, *icon;
+	for (l = applet_list; l; l = g_list_next (l)) {
+		PanelAppletInfo *info;
+		const char *iid, *name, *description, *icon;
 		PanelAddtoItemInfo *applet;
 
-		info = &applet_list->_buffer[i];
+		info = (PanelAppletInfo *)l->data;
 
-		name = bonobo_server_info_prop_lookup (info,
-						       "name",
-						       langs_gslist);
-		description = bonobo_server_info_prop_lookup (info,
-							      "description",
-							      langs_gslist);
-		icon = bonobo_server_info_prop_lookup (info,
-						       "panel:icon",
-						       NULL);
+		iid = panel_applet_info_get_iid (info);
+		name = panel_applet_info_get_name (info);
+		description = panel_applet_info_get_description (info);
+		icon = panel_applet_info_get_icon (info);
 
-		if (!name ||
-		    panel_lockdown_is_applet_disabled (info->iid)) {
+		if (!name || panel_lockdown_is_applet_disabled (iid)) {
 			continue;
 		}
 
@@ -450,14 +409,13 @@ panel_addto_query_applets (GSList *list)
 		applet->name = g_strdup (name);
 		applet->description = g_strdup (description);
 		applet->icon = g_strdup (icon);
-		applet->iid = g_strdup (info->iid);
+		applet->iid = g_strdup (iid);
 		applet->static_data = FALSE;
 
 		list = g_slist_prepend (list, applet);
 	}
 
-	g_slist_free (langs_gslist);
-	CORBA_free (applet_list);
+	g_list_free (applet_list);
 
 	return list;
 }
@@ -612,7 +570,7 @@ panel_addto_prepend_entry (GSList         **parent_list,
 	data = g_new0 (PanelAddtoAppList, 1);
 
 	data->item_info.type          = PANEL_ADDTO_LAUNCHER;
-	data->item_info.name          = g_strdup (gmenu_tree_entry_get_name (entry));
+	data->item_info.name          = g_strdup (gmenu_tree_entry_get_display_name (entry));
 	data->item_info.description   = g_strdup (gmenu_tree_entry_get_comment (entry));
 	data->item_info.icon          = g_strdup (gmenu_tree_entry_get_icon (entry));
 	data->item_info.launcher_path = g_strdup (gmenu_tree_entry_get_desktop_file_path (entry));
@@ -740,6 +698,7 @@ panel_addto_make_application_model (PanelAddtoDialog *dialog)
 				    G_TYPE_STRING);
 
 	tree = gmenu_tree_lookup ("applications.menu", GMENU_TREE_FLAGS_NONE);
+	gmenu_tree_set_sort_key (tree, GMENU_TREE_SORT_DISPLAY_NAME);
 
 	if ((root = gmenu_tree_get_root_directory (tree))) {
 		panel_addto_make_application_list (&dialog->application_list,
@@ -752,6 +711,7 @@ panel_addto_make_application_model (PanelAddtoDialog *dialog)
 	gmenu_tree_unref (tree);
 
 	tree = gmenu_tree_lookup ("settings.menu", GMENU_TREE_FLAGS_NONE);
+	gmenu_tree_set_sort_key (tree, GMENU_TREE_SORT_DISPLAY_NAME);
 
 	if ((root = gmenu_tree_get_root_directory (tree))) {
 		GtkTreeIter iter;
@@ -849,7 +809,7 @@ panel_addto_dialog_response (GtkWidget *widget_dialog,
 	switch (response_id) {
 	case GTK_RESPONSE_HELP:
 		panel_show_help (gtk_window_get_screen (GTK_WINDOW (dialog->addto_dialog)),
-				 "user-guide.xml", "gospanel-15");
+				 "user-guide", "gospanel-15", NULL);
 		break;
 
 	case PANEL_ADDTO_RESPONSE_ADD:
@@ -1131,10 +1091,10 @@ panel_addto_filter_func (GtkTreeModel *model,
 	    gtk_tree_store_iter_depth (GTK_TREE_STORE (model), iter) == 0)
 		return TRUE;
 
-	return (panel_util_utf8_strstrcase (data->name,
-					    dialog->search_text) != NULL ||
-	        panel_util_utf8_strstrcase (data->description,
-					    dialog->search_text) != NULL);
+	return (panel_g_utf8_strstrcase (data->name,
+					 dialog->search_text) != NULL ||
+	        panel_g_utf8_strstrcase (data->description,
+					 dialog->search_text) != NULL);
 }
 
 static void
@@ -1290,6 +1250,7 @@ static PanelAddtoDialog *
 panel_addto_dialog_new (PanelWidget *panel_widget)
 {
 	PanelAddtoDialog *dialog;
+	GtkWidget *dialog_vbox;
 	GtkWidget *vbox;
 	GtkWidget *inner_vbox;
 	GtkWidget *find_hbox;
@@ -1333,7 +1294,10 @@ panel_addto_dialog_new (PanelWidget *panel_widget)
 					 PANEL_ADDTO_RESPONSE_ADD);
 
 	gtk_container_set_border_width (GTK_CONTAINER (dialog->addto_dialog), 5);
-	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog->addto_dialog)->vbox), 2);
+
+	dialog_vbox = gtk_dialog_get_content_area (GTK_DIALOG (dialog->addto_dialog));
+	gtk_box_set_spacing (GTK_BOX (dialog_vbox), 2);
+
 	g_signal_connect (G_OBJECT (dialog->addto_dialog), "response",
 			  G_CALLBACK (panel_addto_dialog_response), dialog);
 	g_signal_connect (dialog->addto_dialog, "destroy",
@@ -1341,8 +1305,7 @@ panel_addto_dialog_new (PanelWidget *panel_widget)
 
 	vbox = gtk_vbox_new (FALSE, 12);
 	gtk_container_set_border_width (GTK_CONTAINER (vbox), 5);
-	gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog->addto_dialog)->vbox),
-			   vbox);
+	gtk_container_add (GTK_CONTAINER (dialog_vbox), vbox);
 
 	inner_vbox = gtk_vbox_new (FALSE, 6);
 	gtk_box_pack_start (GTK_BOX (vbox), inner_vbox, TRUE, TRUE, 0);
